package ru.dbotthepony.kstarbound.util

import ru.dbotthepony.kstarbound.Starbound

class HistoryQueue<T>(val timeToLive: Double, val capacity: Int) {
	constructor(normalTickCapacity: Int) : this(Starbound.TIMESTEP * normalTickCapacity, normalTickCapacity)

	init {
		require(capacity > 0) { "Invalid capacity: $capacity" }
		require(timeToLive > 0.0) { "Invalid time to live: $timeToLive" }
	}

	private val entries = ArrayDeque<Entry<T>>()
	private var time = 0.0
	private var index = 0L

	private data class Entry<T>(val value: T, val time: Double, val index: Long)

	fun add(entry: T) {
		entries.add(Entry(entry, time, index++))

		while (entries.size > capacity) {
			entries.removeFirst()
		}
	}

	fun tick(delta: Double) {
		this.time += delta

		while (entries.isNotEmpty() && this.time - entries.first().time > timeToLive) {
			entries.removeFirst()
		}
	}

	fun query(since: Long = 0L): Pair<List<T>, Long> {
		// TODO: binary search?
		val results = ArrayList<T>()

		entries.forEach {
			if (it.index >= since) {
				results.add(it.value)
			}
		}

		return results to index
	}

	fun clear() {
		time = 0.0
		entries.clear()
	}
}
